package xyz.lychee.lagfixer.support;

import io.papermc.paper.threadedregions.scheduler.ScheduledTask;
import java.lang.reflect.Constructor;
import java.util.List;
import java.util.concurrent.TimeUnit;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.PluginCommand;
import org.bukkit.command.TabCompleter;
import org.bukkit.plugin.Plugin;
import org.bukkit.scheduler.BukkitTask;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.objects.AbstractFork;

public class Folia extends AbstractFork {
   public Folia(Plugin plugin) {
      super(plugin);
   }

   public boolean isSupported() {
      try {
         Class.forName("io.papermc.paper.threadedregions.scheduler.AsyncScheduler");
         return true;
      } catch (ClassNotFoundException var2) {
         return false;
      }
   }

   public int getPriority() {
      return 1;
   }

   public double getMspt() {
      return Bukkit.getAverageTickTime();
   }

   public PluginCommand registerCommand(Plugin plugin, String name, List<String> alliases, CommandExecutor executor) {
      try {
         Constructor<PluginCommand> declaredConstructor = PluginCommand.class.getDeclaredConstructor(String.class, Plugin.class);
         declaredConstructor.setAccessible(true);
         PluginCommand cmd = (PluginCommand)declaredConstructor.newInstance(name, plugin);
         cmd.setAliases(alliases);
         cmd.setExecutor(executor);
         if (executor instanceof TabCompleter) {
            cmd.setTabCompleter((TabCompleter)executor);
         }

         Bukkit.getCommandMap().register(name, cmd);
         return cmd;
      } catch (Exception var7) {
         return null;
      }
   }

   public BukkitTask runSync(@Nullable Location loc, Runnable run) {
      return !this.getPlugin().isEnabled() ? new FoliaTask(this.getPlugin(), (ScheduledTask)null) : new FoliaTask(this.getPlugin(), loc == null ? Bukkit.getGlobalRegionScheduler().run(this.getPlugin(), (s) -> {
         run.run();
      }) : Bukkit.getRegionScheduler().run(this.getPlugin(), loc, (s) -> {
         run.run();
      }));
   }

   public BukkitTask runLater(boolean async, Runnable run, long delayInMs) {
      return this.checkTask(delayInMs) ? new FoliaTask(this.getPlugin(), (ScheduledTask)null) : new FoliaTask(this.getPlugin(), async ? Bukkit.getAsyncScheduler().runDelayed(this.getPlugin(), (s) -> {
         run.run();
      }, delayInMs, TimeUnit.MILLISECONDS) : Bukkit.getGlobalRegionScheduler().runDelayed(this.getPlugin(), (s) -> {
         run.run();
      }, delayInMs / 50L));
   }

   public BukkitTask runTimer(boolean async, Runnable run, long initialDelayInMs, long delayInMs) {
      return this.checkTask(initialDelayInMs, delayInMs) ? new FoliaTask(this.getPlugin(), (ScheduledTask)null) : new FoliaTask(this.getPlugin(), async ? Bukkit.getAsyncScheduler().runAtFixedRate(this.getPlugin(), (s) -> {
         run.run();
      }, initialDelayInMs, delayInMs, TimeUnit.MILLISECONDS) : Bukkit.getGlobalRegionScheduler().runAtFixedRate(this.getPlugin(), (s) -> {
         run.run();
      }, initialDelayInMs / 50L, delayInMs / 50L));
   }

   public BukkitTask runLater(boolean async, Runnable run, long delay, TimeUnit unit) {
      return this.checkTask(delay) ? new FoliaTask(this.getPlugin(), (ScheduledTask)null) : new FoliaTask(this.getPlugin(), async ? Bukkit.getAsyncScheduler().runDelayed(this.getPlugin(), (s) -> {
         run.run();
      }, delay, unit) : Bukkit.getGlobalRegionScheduler().runDelayed(this.getPlugin(), (s) -> {
         run.run();
      }, unit.toMillis(delay) / 50L));
   }

   public BukkitTask runTimer(boolean async, Runnable run, long initialDelay, long delay, TimeUnit unit) {
      return this.checkTask(initialDelay, delay) ? new FoliaTask(this.getPlugin(), (ScheduledTask)null) : new FoliaTask(this.getPlugin(), async ? Bukkit.getAsyncScheduler().runAtFixedRate(this.getPlugin(), (s) -> {
         run.run();
      }, initialDelay, delay, unit) : Bukkit.getGlobalRegionScheduler().runAtFixedRate(this.getPlugin(), (s) -> {
         run.run();
      }, unit.toMillis(initialDelay) / 50L, unit.toMillis(delay) / 50L));
   }

   public static class FoliaTask implements BukkitTask {
      private final @Nullable ScheduledTask task;
      private final Plugin plugin;

      public FoliaTask(Plugin plugin, @Nullable ScheduledTask task) {
         this.task = task;
         this.plugin = plugin;
      }

      public int getTaskId() {
         return this.task == null ? -1 : this.task.hashCode();
      }

      public @NotNull Plugin getOwner() {
         return this.plugin;
      }

      public boolean isSync() {
         return false;
      }

      public boolean isCancelled() {
         return this.task == null || this.task.isCancelled();
      }

      public void cancel() {
         if (this.task != null) {
            this.task.cancel();
         }

      }
   }
}
